package prodcons;

import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class SynchronizedBuffer extends Observable implements Buffer {

    public static final int CAPACITY = 6;
    private Lock accessLock = new ReentrantLock();
    private Condition canWrite = accessLock.newCondition();
    private Condition canRead = accessLock.newCondition();
    private List<Integer> buffer = new LinkedList<Integer>();

    public void set(int value) {
        accessLock.lock();
        try {
            while (buffer.size() == CAPACITY) {
                canWrite.await();
            }
            buffer.add(value);
            updateState();
            canRead.signal();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            accessLock.unlock();
        }
    }

    public int get() {
        int readValue = 0;
        accessLock.lock();
        try {
            while (buffer.size() == 0) {
                canRead.await();
            }
            readValue = buffer.remove(0);
            updateState();
            canWrite.signal();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            accessLock.unlock();
        }
        return readValue;
    }

    private void updateState() {
        setChanged();
        notifyObservers();
    }

    public int[] getValues() {
        int[] retval;
        accessLock.lock();
        try {
            retval = new int[buffer.size()];
            for (int i = 0; i < retval.length; i++) {
                retval[i] = buffer.get(i);
            }
        } finally {
            accessLock.unlock();
        }
        return retval;
    }
}
